from .info import game_info

from .listing import (
    list_3ds_games,
    list_switch_games,
    list_wiiu_games,
)

from .search import (
    search_3ds_games,
    search_switch_games,
    search_wiiu_games,
)
